/* ARCADEmini simulator DLL
 * version 1.3 date 2004-03-15
 * part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
 * Copyright (C) 2003-2004 sphaera <sphaera@geekness.de>
 *                     and 1stein <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

//ARCADEmini device

#ifndef INC_am_dev
#define INC_am_dev

#include <stdio.h>
#include <time.h>

//structure for a state of an ARCADEmini device
typedef struct t_am_state
{
  //file descriptor of file with sectors
  FILE * p_file;
  //file descriptor of file to log to
  FILE * p_log_file;
  //number of sector currently loaded
  //(the code segment register of ARCADEmini)
  unsigned long sector_no;
  //the contens of the current sector
  //(the code segment of ARCADEmini)
  unsigned char sector[0x200];
  //overflow buffer
  // - must always be filled with 0x00
  // - for the case that a long command extends behind end of sector
  unsigned char sector_ov[0x80];
  //current position within current sector
  //(the instruction pointer of ARCADEmini)
  // - only values 0x0..0x1FF are allowed
  unsigned short offset;
  //variables for commands
  //(the general purpose registers of ARCADEmini)
  unsigned char vars[0x100];
  //stack and stack pointer
  // - stack pointer points to location to place next pushed value in
  // - stack pointer is increased after push / decreased before pop
  // - stack wraps around on overflow / underflow
  unsigned char stack[0x80];
  unsigned char stack_ptr;
  //the pixel and overlay buffer
  // - only values 0x00..0x0F are allowed
  // - indices are pixels[y][x], overlay[y][x]
  unsigned char pixels[20][26];
  unsigned char overlay[20][26];
  //the picture currently being shown
  // - only values 0x00..0x0F are allowed
  // - indices are picture[y][x]
  unsigned char picture[20][26];
  //boolean flag if the picture changed
  // - set by am_dev when picture changed
  // - reset by user when picture was read
  unsigned char picture_changed;
  //tick count when the delay currently elapsing is over
  //(points back to past if no delay is active)
  unsigned long delay_end;
  //flag if ot wait for delay
  // - set by DELAY_DO and DELAY_WAIT to indicate that next
  //   command shall not be executed before end of delay
  unsigned char wait_for_delay;
  //state of the keys
  unsigned char keys_current; //current state
  unsigned char keys_pressed; //keys that were pressed since last GET_KEYS
  unsigned char keys_released; //keys that were released sind last GET_KEYS
  //state of the snes keys
  unsigned short snes0_current, snes1_current; //current state
  unsigned short snes0_pressed, snes1_pressed; //keys that were pressed since last GET_SNESx
  unsigned short snes0_released, snes1_released; //keys that were released sind last GET_SNESx
  //time offset in seconds for internal clock (offset from system time)
  //(for not needing to set the system time whwn SET_TIME is executed)
  unsigned long time_offset;
} st_am_state;

//-------------------------------------------------------------------------------------------
// initialize an ARCADEmini state
// returns 0 on success
//-------------------------------------------------------------------------------------------
int am_dev_init_state( st_am_state * p_state, FILE * p_file, FILE * p_log_file );

//-------------------------------------------------------------------------------------------
// execute an ARCADEmini command
// (returns number of milliseconds left of current delay or 0 if not waiting for a delay)
//-------------------------------------------------------------------------------------------
unsigned short am_dev_exec( st_am_state * p_state );

//-------------------------------------------------------------------------------------------
// set a new state of the 8 keys
//-------------------------------------------------------------------------------------------
void am_dev_keystate( st_am_state * p_state, unsigned char keytstate );

//-------------------------------------------------------------------------------------------
// set a new state of the snes keys
//-------------------------------------------------------------------------------------------
void am_dev_snes_keystate( st_am_state * p_state, unsigned char snes_no, unsigned short keystate );

#endif //#ifndef INC_am_dev

